==> trip-0.1.0.tar.gz: trip-0.1.0/util/NetworkAddress.cxx <==
static const char* const NetworkAddress_cxx_fileVersion =
    "$Id: NetworkAddress.cxx,v 1.33 2000/10/02 17:55:37 veer Exp $";


/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
/*
#include <unistd.h>
#include <string.h>
*/

#include <string>
#if defined(__FreeBSD__)
#include <sys/types.h>
#include "netdb_r.h"
#endif
#include <netinet/in.h>
#include <arpa/inet.h>
#include <strstream.h>
#include <netdb.h>
#include "NetworkAddress.h"
#include "cpLog.h"
#include "vsock.hxx"
#include "LockHelper.hxx"
#include <iostream>

VMutex NetworkAddress::_mutex;

//------------------------------------------------------------------------------
// NetworkAddress construstor
//
NetworkAddress::NetworkAddress ( int port )
{
    /* The host is undefined yet.*/
    setHostName("localhost");   // assuming it is localhost first
    aPort = port;
}


NetworkAddress::NetworkAddress ( string hostName, int port /* = -1 */)
{
    setPort(port);
    setHostName(hostName);
}


//------------------------------------------------------------------------------
//NetworkAddress set value method
//
void
NetworkAddress::setHostName ( const string& theAddress )
{
    const char* cstrAddress;
    struct hostent * pxHostEnt = 0;
    string sAddress, sPort;
    unsigned int length = theAddress.length();
    unsigned int colonPos = length;

    cstrAddress = theAddress.c_str();



    for (int i = 0; *cstrAddress != '\0'; cstrAddress++, i++)
    {
        if (*cstrAddress == ':')
        {
            colonPos = i;
            break;
        }
    }

    if (colonPos != length)
    {
        //        cpLog (LOG_INFO, "There is a colon\n");
        sAddress = theAddress.substr(0, colonPos);
        sPort = theAddress.substr(colonPos + 1);

        cpLog (LOG_INFO, "Address[%s]\n", sAddress.c_str());
        cpLog (LOG_INFO, "Port[%s]\n", sPort.c_str());

        colonPos = atoi(sPort.c_str());
        if (colonPos)
        {
            aPort = colonPos;
            cpLog (LOG_INFO, "aPort = %d\n", aPort);
        }
        else
        {
            cpLog (LOG_ERR, "Incorrect Port" );

            cerr << "Incorrect Port" << endl;

        }
    }
    else // No ':' in input string;
    {
        //        cpLog (LOG_INFO, "There is not a colon\n");
        sAddress = theAddress;
    }

    LockHelper helper(_mutex);
    pxHostEnt = gethostbyname(sAddress.c_str());

    if (pxHostEnt)
    {
        memcpy((void *)&ipAddress,
               (void *)pxHostEnt->h_addr,
               pxHostEnt->h_length);
    }
    else
    {
        //        cpLog (LOG_ERR, "Incorrect Address" );


    }

}


void
NetworkAddress::setPort( int iPort )
{
    aPort = iPort;
}


//------------------------------------------------------------------------------
//NetworkAddress get value method
//
string
NetworkAddress::getHostName( ) const
{
    string hostName;
    struct hostent * pxHostEnt = 0;

    _mutex.lock();

    pxHostEnt = gethostbyaddr(ipAddress, IPV4_LENGTH, AF_INET);

    if (pxHostEnt)
    {
        hostName = pxHostEnt->h_name;
        _mutex.unlock();
    }
    else
    {
        _mutex.unlock();
        hostName = getIpName();
    }

    return hostName;
}


string
NetworkAddress::getIpName () const
{
    string ipName;
    struct in_addr temp;

    LockHelper helper(_mutex);

    memcpy((void *)&temp.s_addr, ipAddress, IPV4_LENGTH);
    ipName = inet_ntoa(temp);

    return ipName;
}


// This method is only good for ipv4
// It returns Ip address in network byte order
u_int32_t
NetworkAddress::getIp4Address () const
{
    u_int32_t lTmp;

    memcpy((void *) &lTmp, ipAddress, IPV4_LENGTH);
    return lTmp;
}

void
NetworkAddress::getSockAddr (struct sockaddr & socka) const
{
    short tmp_s = htons (aPort);
    char * tmp_p;

    tmp_p = (char*) &tmp_s;
    socka.sa_family = AF_INET;
    socka.sa_data[0] = tmp_p[0];
    socka.sa_data[1] = tmp_p[1];
    memcpy((void *)&socka.sa_data[2], ipAddress, IPV4_LENGTH);

    return ;
}


int
NetworkAddress::getPort () const
{
    return aPort;
}


//NetworkAddress address to char translation
//
/*
void NetworkAddress::addressToChar ( char * pcAddress ) const
{
    memcpy(pcAddress,m_pcAddress,4);
}
*/

//NetworkAddress operator []
//
/*
unsigned char NetworkAddress::operator [] ( int nth ) const
{
    return m_pcAddress[nth];
}
*/

//NetworkAddress::operator < - Compare two NetworkAddresses
//
bool
operator < ( const NetworkAddress & xAddress,
             const NetworkAddress & yAddress )
{
    for (int i = 0 ; i <= IPV4_LENGTH; i++ )
    {
        if ( xAddress.ipAddress[i] < yAddress.ipAddress[i])
        {
            return true;
        }
        if ( xAddress.ipAddress[i] > yAddress.ipAddress[i])
        {
            return false;
        }
    }
    return xAddress.aPort < yAddress.aPort ;
}

//NetworkAddress::operator == - Compare two NetworkAddresses
//
bool
operator == ( const NetworkAddress & xAddress,
              const NetworkAddress & yAddress )
{
    for (int i = 0 ; i < IPV4_LENGTH ; i++ )
    {
        if ( xAddress.ipAddress[i] != yAddress.ipAddress[i])
        {
            return false;
        }
    }
    return xAddress.aPort == yAddress.aPort ;
}

/// assignment operator =
NetworkAddress& NetworkAddress::operator=( const NetworkAddress& x )
{
    this->aPort = x.aPort;
    memcpy( this->ipAddress, x.ipAddress, sizeof(this->ipAddress) );

    //   strncpy( this->ipAddress, x.ipAddress, IPV4_LENGTH );
    return ( *this );
}


//NetworkAddress typecast to std::string
//
NetworkAddress::operator string ()
{
    string xStr;
    strstream xStream;
    xStream << (*this);
    xStr = xStream.str();
    xStream.freeze(false);
    return xStr;
}


ostream &
NetworkAddress::print ( ostream & xStr ) const
{
    struct hostent * pxHostEnt;
    struct in_addr temp;
    memcpy((void *)&temp.s_addr, ipAddress, IPV4_LENGTH);
    char * cpBuf = (char *)inet_ntoa(temp);

    LockHelper helper(_mutex);

    pxHostEnt = gethostbyaddr(ipAddress, IPV4_LENGTH, AF_INET);

    if ( pxHostEnt )
    {
        xStr << pxHostEnt->h_name << "(" << cpBuf << ")";
    }
    else
    {
        xStr << cpBuf ;
    }
    if ( aPort )
        xStr << ":" << aPort;

    return xStr;
}

///
u_int32_t
NetworkAddress::hashIpPort( )
{
    u_int32_t ipAddressLocal = getIp4Address ();
    int port = getPort ();
    return (hashIpPort( ipAddressLocal, port )) ;
}

u_int32_t
NetworkAddress::hashIpPort( const string& theAddress, const string& port )
{
    static NetworkAddress net;

    net.setHostName ( theAddress );
    net. setPort ( atoi(port.c_str()) );

    return ( net.hashIpPort() );
}

u_int32_t
NetworkAddress::hashIpPort( const u_int32_t ipAddress, const int port )
{
    u_int32_t hashKey = 0x0;
    hashKey = (ipAddress << 16) | (port & 0xFF);

    return hashKey;
}

int
NetworkAddress::getHostByName(const char* hostName,
                              struct hostent* hEnt,
                              char* buf,
                              int buflen,
                              int* thrErrno)
{
#if defined(__linux__)
    struct hostent* x;
    int retval = gethostbyname_r(hostName, hEnt, buf, buflen, &x, thrErrno);
    if (retval == 0)
        return getHostLookupOK;

    cpLog(LOG_DEBUG, "********* Failed to get host by name:%s", hostName);

    return getHostLookupFailed;

#else
#if defined(__svr4__) || defined(__FreeBSD__)|| defined (__SUNPRO_CC)
    struct hostent* x;
    x = gethostbyname_r(hostName, hEnt, buf, buflen, thrErrno);
    if (x == 0)
    {
        cpLog(LOG_DEBUG, "********* Failed to get host by name:%s", hostName);
        return getHostLookupFailed;
    }
    return getHostLookupOK;
#else
#error no implementation for critical function
#endif
#endif

}


// End of File

==> trip-1.0.0.tar.gz: trip-1.0.0/util/NetworkAddress.cxx <==
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const NetworkAddress_cxx_Version =
    "$Id: NetworkAddress.cxx,v 1.39 2001/03/12 23:53:34 maneri Exp $";

#include <string>
#if defined(__FreeBSD__)
#include <sys/types.h>
#include "netdb_r.h"
#endif
#include <netinet/in.h>
#include <arpa/inet.h>
#include <strstream.h>
#include <netdb.h>
#include "NetworkAddress.h"
#include "cpLog.h"
#include "vsock.hxx"
#include "LockHelper.hxx"
#include <iostream>

VMutex NetworkAddress::mutex_;


NetworkAddress::NetworkAddress ( int port /* = -1 */) 
   : ipAddressSet(false)
{
    setHostName("localhost");   // assuming it is localhost first
    aPort = port;
}


NetworkAddress::NetworkAddress ( string hostName, int port /* = -1 */)
    : ipAddressSet(false)
{
    setPort(port);
    setHostName(hostName);
}


void
NetworkAddress::setHostName ( const string& theAddress )
{
    const char* cstrAddress;
    string sAddress, sPort;
    unsigned int length = theAddress.length();
    unsigned int colonPos = length;

    cstrAddress = theAddress.c_str();

    for (int i = 0; *cstrAddress != '\0'; cstrAddress++, i++)
    {
        if (*cstrAddress == ':')
        {
            colonPos = i;
            break;
        }
    }

    if (colonPos != length)
    {
        sAddress = theAddress.substr(0, colonPos);
        sPort = theAddress.substr(colonPos + 1);

        cpLog (LOG_INFO, "Address[%s]\n", sAddress.c_str());
        cpLog (LOG_INFO, "Port[%s]\n", sPort.c_str());

        colonPos = atoi(sPort.c_str());
        if (colonPos)
        {
            aPort = colonPos;
            cpLog (LOG_INFO, "aPort = %d\n", aPort);
        }
        else
        {
            cpLog (LOG_ERR, "Incorrect Port" );

            cerr << "Incorrect Port" << endl;
        }
    }
    else // No ':' in input string;
    {
        sAddress = theAddress;
    }
#ifdef __linux__
    rawHostName = sAddress;
#else
    // since this is not thread safe on linux, we only want to do this
    // on solaris.
    rawHostName.assign(sAddress.c_str(), sAddress.length());
#endif
    ipAddressSet = false;
}


void
NetworkAddress::setPort( int iPort )
{
    ipAddressSet = false;
    aPort = iPort;
}


string
NetworkAddress::getHostName( ) const
{
    string hostName;
    struct hostent * pxHostEnt = 0;

    initIpAddress();

    mutex_.lock();

#if defined(__linux__)
    pxHostEnt = gethostbyaddr(ipAddress, IPV4_LENGTH, AF_INET);
#else
    int err_num;
    pxHostEnt = getipnodebyaddr(ipAddress, AF_INET, 0, &err_num);
#endif

    if (pxHostEnt)
    {
        hostName = pxHostEnt->h_name;
        mutex_.unlock();
    }
    else
    {
        mutex_.unlock();
        hostName = getIpName();
    }

    return hostName;
}


string
NetworkAddress::getIpName () const
{
    string ipName;
    struct in_addr temp;

    initIpAddress();

    memcpy((void *)&temp.s_addr, ipAddress, IPV4_LENGTH);
    LockHelper helper(mutex_);
    ipName = inet_ntoa(temp);

    return ipName;
}


u_int32_t
NetworkAddress::getIp4Address () const
{
    u_int32_t lTmp;
    initIpAddress();
    memcpy((void *) &lTmp, ipAddress, IPV4_LENGTH);
    return lTmp;
}


void
NetworkAddress::getSockAddr (struct sockaddr & socka) const
{
    short tmp_s = htons (aPort);
    char * tmp_p;

    tmp_p = (char*) &tmp_s;
    socka.sa_family = AF_INET;
    socka.sa_data[0] = tmp_p[0];
    socka.sa_data[1] = tmp_p[1];
    initIpAddress();
    memcpy((void *)&socka.sa_data[2], ipAddress, IPV4_LENGTH);

    return ;
}


int
NetworkAddress::getPort () const
{
    return aPort;
}


bool
operator < ( const NetworkAddress & xAddress,
             const NetworkAddress & yAddress )
{
    xAddress.initIpAddress();
    yAddress.initIpAddress();
    for (int i = 0 ; i <= IPV4_LENGTH; i++ )
    {
        if ( xAddress.ipAddress[i] < yAddress.ipAddress[i])
        {
            return true;
        }
        if ( xAddress.ipAddress[i] > yAddress.ipAddress[i])
        {
            return false;
        }
    }
    return xAddress.aPort < yAddress.aPort ;
}


bool
operator == ( const NetworkAddress & xAddress,
              const NetworkAddress & yAddress )
{
    xAddress.initIpAddress();
    yAddress.initIpAddress();
    for (int i = 0 ; i < IPV4_LENGTH ; i++ )
    {
        if ( xAddress.ipAddress[i] != yAddress.ipAddress[i])
        {
            return false;
        }
    }
    return xAddress.aPort == yAddress.aPort ;
}


bool
operator != ( const NetworkAddress & xAddress,
              const NetworkAddress & yAddress )
{
    return !(xAddress == yAddress);
}


NetworkAddress& 
NetworkAddress::operator=( const NetworkAddress& x )
{
    this->aPort = x.aPort;
    memcpy( this->ipAddress, x.ipAddress, sizeof(this->ipAddress) );
    ipAddressSet = x.ipAddressSet;
    rawHostName = x.rawHostName;
    //   strncpy( this->ipAddress, x.ipAddress, IPV4_LENGTH );
    return ( *this );
}

NetworkAddress::NetworkAddress( const NetworkAddress& x)
{
    aPort = x.aPort;
    ipAddressSet = x.ipAddressSet;
    rawHostName = x.rawHostName;
    memcpy( ipAddress, x.ipAddress, sizeof(ipAddress) );
}


NetworkAddress::operator string ()
{
    string xStr;
    strstream xStream;
    xStream << (*this);
    xStr = xStream.str();
    xStream.freeze(false);
    return xStr;
}


ostream &
NetworkAddress::print ( ostream & xStr ) const
{
    struct hostent * pxHostEnt;
    struct in_addr temp;
    initIpAddress();
    memcpy((void *)&temp.s_addr, ipAddress, IPV4_LENGTH);
    char * cpBuf = (char *)inet_ntoa(temp);

    LockHelper helper(mutex_);

    pxHostEnt = gethostbyaddr(ipAddress, IPV4_LENGTH, AF_INET);

    if ( pxHostEnt )
    {
        xStr << pxHostEnt->h_name << "(" << cpBuf << ")";
    }
    else
    {
        xStr << cpBuf ;
    }
    if ( aPort )
        xStr << ":" << aPort;

    return xStr;
}


u_int32_t
NetworkAddress::hashIpPort( )
{
    u_int32_t ipAddressLocal = getIp4Address ();
    int port = getPort ();
    return (hashIpPort( ipAddressLocal, port )) ;
}


u_int32_t
NetworkAddress::hashIpPort( const string& theAddress, const string& port )
{
    NetworkAddress net;

    net.setHostName ( theAddress );
    net. setPort ( atoi(port.c_str()) );

    return ( net.hashIpPort() );
}


u_int32_t
NetworkAddress::hashIpPort( const u_int32_t ipAddress, const int port )
{
    u_int32_t hashKey = 0x0;
    hashKey = (ipAddress << 16) | (port & 0xFF);

    return hashKey;
}

void
NetworkAddress::initIpAddress() const
{
    if(!ipAddressSet)
    {
        if (true == is_valid_ip_addr(rawHostName) )
        {
            ipAddressSet = true;
        }
        else
        {
            LockHelper helper(mutex_);
            struct hostent * pxHostEnt = 0;
            pxHostEnt = gethostbyname(rawHostName.c_str());

            if (pxHostEnt)
            {
                memcpy((void *)&ipAddress,
                       (void *)pxHostEnt->h_addr,
                       pxHostEnt->h_length);
                ipAddressSet = true;
            }
            else
            {
                cpLog (LOG_ERR, "Could not resolve Address for (%s)", rawHostName.c_str());
            }
        }
    }
}


int
NetworkAddress::getHostByName(const char* hostName,
                              struct hostent* hEnt,
                              char* buf,
                              int buflen,
                              int* thrErrno)
{
#if defined(__linux__)
    struct hostent* x;
    int retval = gethostbyname_r(hostName, hEnt, buf, buflen, &x, thrErrno);
    if (retval == 0)
        return getHostLookupOK;

    cpLog(LOG_DEBUG, "********* Failed to get host by name:%s", hostName);

    return getHostLookupFailed;

#else
#if defined(__svr4__) || defined(__FreeBSD__)|| defined (__SUNPRO_CC)
    struct hostent* x;
    x = gethostbyname_r(hostName, hEnt, buf, buflen, thrErrno);
    if (x == 0)
    {
        cpLog(LOG_DEBUG, "********* Failed to get host by name:%s", hostName);
        return getHostLookupFailed;
    }
    return getHostLookupOK;
#else
#error no implementation for critical function
#endif
#endif
}


/**
  * Return TRUE if the address is a valid IP address (dot-decimal form)
  * such as 128.128.128.128. Checks for invalid or ill formed addresses 
  * If the address is valid, copy it into ipAddress
  */
bool
NetworkAddress::is_valid_ip_addr(const string& addr) const
{

    ulong maskcheck = ~255; // mask to check if 'tween 0 and 255
    const char *advancer = addr.c_str();
    ulong octet;
    char *nextchar;

    // always check for spaces and right number
    // first and last fields must be 1-255, middle two 0-255
    
    if ((*(advancer) == 0) || (*(advancer) == ' ') || (*(advancer) == '\t'))
    {
        return false;
    }
    octet = strtoul(advancer, &nextchar, 10);
    if((*nextchar != '.') || (octet & maskcheck) || (octet == 0))
    {
        return false;
    }
    ipAddress[0] = octet;


    advancer = nextchar+1;
    if ((*(advancer) == 0) || (*(advancer) == ' ') || (*(advancer) == '\t'))
    {
        return false;
    }
    octet = strtoul(advancer, &nextchar, 10);
    if((*nextchar != '.') || (octet & maskcheck))
    {
        return false;
    }
    ipAddress[1] = octet;


    advancer = nextchar+1;
    if ((*(advancer) == 0) || (*(advancer) == ' ') || (*(advancer) == '\t'))
    {
        return false;
    }
    octet = strtoul(advancer, &nextchar, 10);
    if((*nextchar != '.') || (octet & maskcheck))
    {
        return false;
    }
    ipAddress[2] = octet;
    

    advancer = nextchar+1;
    if ((*(advancer) == 0) || (*(advancer) == ' ') || (*(advancer) == '\t'))
    {
        return false;
    }
    octet = strtoul(advancer, &nextchar, 10);
    if((*nextchar) || (octet & maskcheck) || (octet == 0))
    {
        return false;
    }
    ipAddress[3] = octet;
    
    return true;

}

// End of File
